LIBRARY ieee;
USE ieee.std_logic_1164.all;
USE ieee.std_logic_unsigned.all;

entity part4 is
	port
	(
			CLOCK_50	 : in std_logic;
			KEY : in STD_LOGIC_VECTOR(1 downto 0);	
		   SW  : in STD_LOGIC_VECTOR(2 downto 0);	
			LEDR : out STD_LOGIC_VECTOR(9 downto 0)
	);
end;


architecture behav of part4 is
   CONSTANT A : STD_LOGIC_VECTOR(3 downto 0) := "0001";
   CONSTANT LEN_A : STD_LOGIC_VECTOR(2 downto 0) := "010";
   CONSTANT B : STD_LOGIC_VECTOR(3 downto 0) := "1110";
   CONSTANT LEN_B : STD_LOGIC_VECTOR(2 downto 0) := "100";
   CONSTANT C : STD_LOGIC_VECTOR(3 downto 0) := "1010";
   CONSTANT LEN_C : STD_LOGIC_VECTOR(2 downto 0) := "100";
   CONSTANT D : STD_LOGIC_VECTOR(3 downto 0) := "0110";
   CONSTANT LEN_D : STD_LOGIC_VECTOR(2 downto 0) := "011";
   CONSTANT E : STD_LOGIC_VECTOR(3 downto 0) := "0001";
   CONSTANT LEN_E : STD_LOGIC_VECTOR(2 downto 0) := "001";
   CONSTANT F : STD_LOGIC_VECTOR(3 downto 0) := "1011";
   CONSTANT LEN_F : STD_LOGIC_VECTOR(2 downto 0) := "100";
   CONSTANT G : STD_LOGIC_VECTOR(3 downto 0) := "0100";
   CONSTANT LEN_G : STD_LOGIC_VECTOR(2 downto 0) := "011";
   CONSTANT H : STD_LOGIC_VECTOR(3 downto 0) := "1111";
   CONSTANT LEN_H : STD_LOGIC_VECTOR(2 downto 0) := "100";
	
	type StateType is ( S_WAIT, S_BLANK, S_DOT, S_LINE1, S_LINE2, S_LINE3 );
	
	signal CurrentState, NextState : StateType;
	signal ShiftLetter, SelectedLetter : STD_LOGIC_VECTOR(3 downto 0);
	signal CountLen, SelectedLength : STD_LOGIC_VECTOR(2 downto 0);

   signal clock, resetn, start, en_count_shift, ld_count_shift, pulse_05s, light, resetlatch : STD_LOGIC;

begin

   clock <= CLOCK_50;
	resetn <= KEY(0);
	LEDR(0) <= light;
	LEDR(9 downto 1) <= (others => '0');
	
	process(KEY(1))
	begin
		if (resetlatch = '1') then
			start<='0';
		elsif KEY(1)='0' then
			start<='1';
		end if;
	end process;

Count0:
   entity work.counter(behav)
	generic map(25, 25_000_000)
	port map(clock=> CLOCK_50, reset_n => resetn, enable=>'1', Q => open, rollover => pulse_05s);
	
	
SelectLetter: process (SW) is
   begin
		case SW is
			when "000" =>
				SelectedLetter <= A;
				SelectedLength <= LEN_A;
			when "001" =>
				SelectedLetter <= B;
				SelectedLength <= LEN_B;
			when "010" =>
				SelectedLetter <= C;
				SelectedLength <= LEN_C;				
			when "011" =>
				SelectedLetter <= D;
				SelectedLength <= LEN_D;
			when "100" =>
				SelectedLetter <= E;
				SelectedLength <= LEN_E;				
			when "101" =>
				SelectedLetter <= F;
				SelectedLength <= LEN_F;
			when "110" =>
				SelectedLetter <= G;
				SelectedLength <= LEN_G;
			when "111" =>
				SelectedLetter <= H;
				SelectedLength <= LEN_H;				
			end case;
	end process;
	
	process
	begin
		wait until rising_edge(clock); 
		if resetn='0' then
			ShiftLetter <= "0000";
			CountLen <= "000";
		elsif ld_count_shift='1' then
			ShiftLetter <= SelectedLetter;
			CountLen<=SelectedLength;
		elsif en_count_shift='1' then
			ShiftLetter(2 downto 0) <= ShiftLetter(3 downto 1);
			CountLen<=CountLen-1;
		end if;
	end process;
	
	process(start, pulse_05s, CurrentState, ShiftLetter(0), CountLen)
	begin
		en_count_shift<='0'; 
		ld_count_shift<='0'; 
		light <= '0';
		resetlatch<='0';
		case CurrentState is
			when S_WAIT =>
			   if start='1' then
					NextState<=S_BLANK;
				else
					NextState<=S_WAIT;
				end if;
				ld_count_shift<='1';
			when S_BLANK =>
			   if ShiftLetter(0)='1' then
					NextState<=S_DOT;
				else
					NextState<=S_LINE1;
				end if;
				en_count_shift <= pulse_05s;
			when S_DOT =>	
				if CountLen="000" then
					NextState<=S_WAIT;
					resetlatch<='1';
				else
					NextState<=S_BLANK;
				end if;
				light<='1';
			when S_LINE1 =>
				NextState<=S_LINE2;
				light<='1';
			when S_LINE2 =>
				NextState<=S_LINE3;
				light<='1';
			when S_LINE3 =>	
				if CountLen="000" then
					NextState<=S_WAIT;
					resetlatch<='1';
				else
					NextState<=S_BLANK;
				end if;
				light<='1';
		end case;
	end process;
	
	process(clock, resetn)
	begin
		if (resetn='0') then
			CurrentState<=S_WAIT;
		elsif(rising_edge(clock)) then
			if (pulse_05s='1') then
				CurrentState<=NextState;
			end if;
		end if;
	end process;

end architecture;
	
	
	
